
# https://www.unidata.ucar.edu/software/netcdf/workshops/most-recent/netcdf_python/netcdf4python.pdf

# http://unidata.github.io/netcdf4-python/

# https://github.com/Unidata/netcdf4-python

import netCDF4
import numpy as np

nc = netCDF4.Dataset('test4dz28992.nc', 'w', format='NETCDF4')
nc.Conventions = "CF-1.6"
nc._CoordSysBuilder = "ucar.nc2.dataset.conv.CF1Convention"

# Create DIMENSIONS first
# No need to create a handle/variable for it. Nowhere used, and netcdf lib itself uses the dimname given as 1st param
nc.createDimension('y', 3)
nc.createDimension('x', 4)
nc.createDimension('time', None)  # unlimited
nc.createDimension('z', 3)

# VARIABLES
# over coordinate variabelen
# https://www.unidata.ucar.edu/software/netcdf/docs/netcdf_data_set_components.html#coordinate_variables
# komt er op neer dat de dimensie van deze 'coordinate_variables' dezelfde naam moet hebben????

y = nc.createVariable("y", "d", ("y",))
x = nc.createVariable("x", "d", ("x",))
time = nc.createVariable("time", "f8", ("time",))
z = nc.createVariable("z", "f8", ("z",))
data28992 = nc.createVariable('data28992', np.float32, ('time', 'z', 'y', 'x'))

oblique_stereographic = nc.createVariable("oblique_stereographic", "c")
oblique_stereographic.grid_mapping_name = "oblique_stereographic"
oblique_stereographic.longitude_of_central_meridian = 5.38763888888889
oblique_stereographic.false_easting = 155000.0
oblique_stereographic.false_northing = 463000.0
oblique_stereographic.latitude_of_projection_origin = 52.15616055555555
oblique_stereographic.scale_factor_at_central_meridian = 0.9999079
oblique_stereographic.scale_factor_at_projection_origin = 0.9999079
oblique_stereographic.long_name = "CRS definition"
oblique_stereographic.longitude_of_prime_meridian = 0.0
oblique_stereographic.semi_major_axis = 6377397.155
oblique_stereographic.inverse_flattening = 299.1528128
oblique_stereographic.spatial_ref = "PROJCS[\"Amersfoort / RD New\",GEOGCS[\"Amersfoort\",DATUM[\"Amersfoort\",SPHEROID[\"Bessel 1841\",6377397.155,299.1528128,AUTHORITY[\"EPSG\",\"7004\"]],TOWGS84[565.417,50.3319,465.552,-0.398957,0.343988,-1.8774,4.0725],AUTHORITY[\"EPSG\",\"6289\"]],PRIMEM[\"Greenwich\",0,AUTHORITY[\"EPSG\",\"8901\"]],UNIT[\"degree\",0.0174532925199433,AUTHORITY[\"EPSG\",\"9122\"]],AUTHORITY[\"EPSG\",\"4289\"]],PROJECTION[\"Oblique_Stereographic\"],PARAMETER[\"latitude_of_origin\",52.15616055555555],PARAMETER[\"central_meridian\",5.38763888888889],PARAMETER[\"scale_factor\",0.9999079],PARAMETER[\"false_easting\",155000],PARAMETER[\"false_northing\",463000],UNIT[\"metre\",1,AUTHORITY[\"EPSG\",\"9001\"]],AXIS[\"X\",EAST],AXIS[\"Y\",NORTH],AUTHORITY[\"EPSG\",\"28992\"]]"
oblique_stereographic.GeoTransform = "140000 5 0 462500 0 -5 "
oblique_stereographic._CoordinateTransformType = "Projection"
oblique_stereographic._CoordinateAxisTypes = "GeoX GeoY"
# TODO better proj4 string !!:
# +proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +k=0.9999079 +x_0=155000 +y_0=463000 +ellps=bessel +towgs84=565.2369,50.0087,465.658,-0.406857,0.350733,-1.87035,4.0812 +units=m +no_defs
oblique_stereographic.proj4_params = "\" +proj=sterea +lat_0=52.15616055555555 +lon_0=5.38763888888889 +k=0.9999079 +x_0=155000 +y_0=463000 +ellps=bessel +units=m +no_defs +<>\"";
oblique_stereographic.EPSG_code = "EPSG:28992";


xllcorner = 3.7  # 3.733  #3.416040182
yllcorner = 51.4 # 51.429 #53.40350723

# cells 1000x1000m
y.units = "m"
y.standard_name = "projection_y_coordinate"
y.long_name = "y coordinate of projection"
y._CoordinateAxisType = "GeoY";
y[:] = [380341,381341,382341]

x.units = "m"
x.standard_name = "projection_x_coordinate"
x.long_name = "x coordinate of projection"
x._CoordinateAxisType = "GeoX";
x[:] = [37596,38596,39596,40596]


time.units = "hours since 0001-01-01 00:00:00.0"
time.calendar = "gregorian"
time.standard_name = "time"
time.axis = "time"
time[:] = [17533056., 17533068., 17533080.]
#time[:] = [17533056.,]

z.units = "meters"
z.axis = "Z"
z.positive = "up"
z.standard_name = "height"
z._CoordinateAxisType = "GeoZ";
z[:] = [0, 100, 200]

#
# #latitudes = nc.createVariable("lat", "f4", ("lat",))
# latitudes = nc.createVariable("lat", "f4", ("y","x"))
# latitudes.units = 'degrees_north'
# latitudes.standard_name = 'latitude'
# latitudes.long_name = 'latitude'
# latitudes._CoordinateAxisType = "Lat";
# #longitudes = nc.createVariable("lon", "f4", ("lon",))
# longitudes = nc.createVariable("lon", "f4", ("y","x"))
# longitudes.units = 'degrees_east'
# longitudes.standard_name = 'longitude'
# longitudes.long_name = 'longitude'
# longitudes._CoordinateAxisType = "Lon";

# p as ('time', 'z', 'y', 'x') working in QGIS
p = np.array([
              [
                [[2,3,1,2],[3,1,2,3],[1,2,3,1]], [[4,3,2,2],[2,1,4,3],[1,1,2,1]], [[3,1,2,3],[3,1,2,3],[2,3,1,2]]
              ],
              [
                [[4,3,2,2],[2,1,4,3],[1,1,2,1]], [[3,1,2,3],[3,1,2,3],[2,3,1,2]], [[2,3,1,2],[3,1,2,3],[1,2,3,1]]
              ],
              [
                [[4,3,2,2],[2,1,4,3],[1,1,2,1]], [[2,3,1,2],[3,1,2,3],[1,2,3,1]], [[3,1,2,3],[3,1,2,3],[2,3,1,2]]
              ]
            ])
print(p.shape)  # (3, 3, 3, 4)
# data28992 = nc.createVariable('data28992', np.float32, ('time', 'z', 'y', 'x'))
data28992[0:3, 0:3, 0:3, 0:4] = p


# p2 as ('y', 'x')  working in Geoserver
# p2 = np.array([ [2,3,1,2],[3,1,2,3],[1,2,3,1] ])
# print(p2.shape) # (3, 4)
#data28992 = nc.createVariable('data28992', np.float32, ('y', 'x'))
#data28992[0:3, 0:4] = p2


# p3 as ('time', 'y', 'x')  NOT working in Geoserver
# p3 = np.array([ [[2,3,1,2],[3,1,2,3],[1,2,3,1]], ])
# print(p3.shape) # (1, 3, 4)
# data28992 = nc.createVariable('data28992', np.float32, ('time', 'y', 'x'))
# data28992[0:1, 0:3, 0:4] = p3


data28992.standard_name = 'data28992'
data28992.long_name = 'data in 28992'
#data28992.coordinates = "lon lat"
# onderstaande zijn eigenlijk niet nodig ???
data28992.grid_mapping = "oblique_stereographic"
#data28992.grid_mapping = "projection"

nc.close()





